"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs-extra");
exports.defaultFixturesLocation = path.join(__dirname, '..', '..', 'fixtures');
exports.defaultUpdateFixtures = false;
function create(fixturesLocation = exports.defaultFixturesLocation) {
    const helper = {
        read,
        write,
        compare,
        relativeFixturePath,
        resolveFixturePath,
        fixturesLocation,
        enableWriteBack: exports.defaultUpdateFixtures,
    };
    function relativeFixturePath(...pathParts) {
        return path.join(...pathParts);
    }
    function resolveFixturePath(...pathParts) {
        return path.join(fixturesLocation, ...pathParts);
    }
    function read(...fixturePath) {
        return fs.readFile(resolveFixturePath(...fixturePath), 'utf8');
    }
    async function write(fixturePath, text) {
        const filename = resolveFixturePath(fixturePath);
        await fs.ensureDir(path.dirname(filename));
        return fs.writeFile(filename, text);
    }
    async function compare(fixturePath, actual) {
        if (helper.enableWriteBack) {
            await write(fixturePath, actual);
        }
        const expected = await read(fixturePath);
        return { expected, actual };
    }
    return helper;
}
exports.create = create;
//# sourceMappingURL=fixtures.js.map