import { GrammarDefinition, Pattern } from './grammarDefinition';
import { Token } from './tokenize';
export { Token, tokenizeLine } from './tokenize';
export interface Tokenizer {
    tokenize(line: string): Token[];
}
export interface TokenizeLineResult {
    line: string;
    tokens: Token[];
    lineNumber: number;
}
export interface TokenizeTextResult {
    tokenizedLines: TokenizeLineResult[];
}
export declare type ScopeResolver = (scopeNameRef: string) => Pattern | undefined;
export declare class Grammar {
    private grammarDef;
    constructor(grammarDef: GrammarDefinition);
    tokenizer(): Tokenizer;
    tokenizeLines(input: Iterable<string>): IterableIterator<TokenizeLineResult>;
    readonly grammar: GrammarDefinition;
    /**
     *
     * @param scopeNameRef the name of the scope to look up in the repository. It is of the form
     *                     `scope.ext#reference`
     */
    getReferencePattern(scopeNameRef: string): Pattern | undefined;
    private getPattern(ref);
    resolveImports(resolver: ScopeResolver): void;
    static createFromFile(filename: string): Promise<Grammar>;
}
