"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tokenize_1 = require("./tokenize");
const fs = require("fs-extra");
const pattern_1 = require("./pattern");
var tokenize_2 = require("./tokenize");
exports.tokenizeLine = tokenize_2.tokenizeLine;
class Grammar {
    constructor(grammarDef) {
        this.grammarDef = grammarDef;
        // fixup the repository to include $self and $base and ensure it exists.
        const base = { $self: grammarDef, $base: grammarDef };
        const repository = grammarDef.repository || base;
        Object.assign(repository, base);
        grammarDef.repository = repository;
    }
    tokenizer() {
        let rule = tokenize_1.grammarToRule(this.grammarDef);
        return {
            tokenize: (line) => {
                const r = tokenize_1.tokenizeLine(line, rule);
                rule = r.state;
                return r.tokens;
            }
        };
    }
    *tokenizeLines(input) {
        const tokenizer = this.tokenizer();
        let lineNumber = 0;
        for (const line of input) {
            lineNumber += 1;
            const tokens = tokenizer.tokenize(line);
            yield { line, lineNumber, tokens };
        }
    }
    get grammar() {
        return this.grammarDef;
    }
    /**
     *
     * @param scopeNameRef the name of the scope to look up in the repository. It is of the form
     *                     `scope.ext#reference`
     */
    getReferencePattern(scopeNameRef) {
        const [scope, reference = '$self'] = scopeNameRef.split('#', 2);
        if (scope !== this.grammarDef.scopeName) {
            return undefined;
        }
        return this.getPattern(reference);
    }
    getPattern(ref) {
        const pattern = this.grammarDef.repository[ref];
        return pattern || undefined;
    }
    resolveImports(resolver) {
        const self = this;
        function internalResolve(scopeNameRef) {
            const [scope, reference = ''] = scopeNameRef.split('#', 2);
            return (!scope || scope === '$self' || scope === '$base')
                ? self.getPattern(reference || scope)
                : resolver(scopeNameRef);
        }
        function resolvePatternInclude(pat) {
            if (!pat._reference) {
                const refMaybe = internalResolve(pat.include);
                if (!refMaybe) {
                    console.log(`Cannot resolve reference: (${self.grammarDef.scopeName}:${pat.include})`);
                }
                const ref = refMaybe || { name: pat.include };
                pat._reference = ref;
            }
        }
        function resolvePatterns(pat) {
            if (pattern_1.isPatternInclude(pat)) {
                resolvePatternInclude(pat);
            }
            else if (pattern_1.isPatternPatterns(pat)) {
                (pat.patterns || []).forEach(p => resolvePatterns(p));
            }
        }
        const repository = this.grammarDef.repository;
        resolvePatterns(this.grammarDef);
        Object.keys(repository)
            .map(key => repository[key])
            .forEach(resolvePatterns);
    }
    static async createFromFile(filename) {
        const json = await fs.readFile(filename, 'utf8');
        const grammarDef = JSON.parse(json);
        return new Grammar(grammarDef);
    }
}
exports.Grammar = Grammar;
//# sourceMappingURL=grammar.js.map