"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function isPatternInclude(pattern) {
    const { include = undefined } = pattern;
    return include !== undefined;
}
exports.isPatternInclude = isPatternInclude;
function isPatternMatch(pattern) {
    const { match = undefined } = pattern;
    return match !== undefined;
}
exports.isPatternMatch = isPatternMatch;
function isPatternBeginEnd(pattern) {
    const { begin = undefined, end = undefined } = pattern;
    return begin !== undefined && end !== undefined;
}
exports.isPatternBeginEnd = isPatternBeginEnd;
function isPatternPatterns(pattern) {
    const { patterns = undefined } = pattern;
    return patterns !== undefined;
}
exports.isPatternPatterns = isPatternPatterns;
function isPatternName(pattern) {
    const { name = undefined } = pattern;
    return name !== undefined
        && !isPatternMatch(pattern)
        && !isPatternBeginEnd(pattern)
        && !isPatternInclude(pattern)
        && !isPatternPatterns(pattern);
}
exports.isPatternName = isPatternName;
function scope(pattern) {
    if (isPatternMatch(pattern) || isPatternBeginEnd(pattern)) {
        return pattern.name;
    }
    if (isPatternPatterns(pattern)) {
        return pattern.scopeName;
    }
    return undefined;
}
exports.scope = scope;
function captures(pattern) {
    if (isPatternMatch(pattern)) {
        return pattern.captures;
    }
    if (isPatternBeginEnd(pattern)) {
        return pattern.beginCaptures || pattern.captures;
    }
    return undefined;
}
exports.captures = captures;
function endCaptures(pattern) {
    if (isPatternBeginEnd(pattern)) {
        return pattern.endCaptures || pattern.captures;
    }
    return undefined;
}
exports.endCaptures = endCaptures;
function patternToString(pattern) {
    if (isPatternMatch(pattern)) {
        return `PatternMatch: ${pattern.name || '?'} (${pattern.match.toString()})`;
    }
    if (isPatternBeginEnd(pattern)) {
        return `PatternBeginEnd: ${pattern.name || '?'} (${pattern.begin.toString()})`;
    }
    if (isPatternPatterns(pattern)) {
        return `PatternPatterns: ${pattern.name || '?'} [${pattern.patterns.length}]`;
    }
    if (isPatternInclude(pattern)) {
        return `PatternInclude: ${pattern.name || '?'} (${pattern.include})`;
    }
    // pattern name
    return `PatternName: ${pattern.name || '?'}`;
}
exports.patternToString = patternToString;
//# sourceMappingURL=pattern.js.map