"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const grammar_1 = require("./grammar");
const grammarFiles_1 = require("./grammarFiles");
class Registry {
    constructor(grammars) {
        this.grammars = grammars;
        this.scopeMap = new Map();
        this.fileTypeMap = new Map();
        this.grammars.forEach(g => this.scopeMap.set(g.grammar.scopeName, g));
        this.grammars.forEach(g => this.fileTypeMap.set(g.grammar.scopeName.replace(/^.*\./, ''), g));
        this.grammars.forEach(g => (g.grammar.fileTypes || [])
            .map(t => this.normalizeFileType(t))
            .forEach(t => this.fileTypeMap.set(t, g)));
        this.grammars.forEach(g => g.resolveImports(scopeNameRef => {
            const [scope] = scopeNameRef.split('#', 2);
            const refGrammar = this.scopeMap.get(scope);
            return refGrammar && refGrammar.getReferencePattern(scopeNameRef);
        }));
    }
    normalizeFileType(fileType) {
        return fileType.replace(/^\./, '');
    }
    getGrammarForFileType(fileType) {
        return this.fileTypeMap.get(this.normalizeFileType(fileType));
    }
    static async create(grammarFileNames) {
        const pGrammars = grammarFileNames
            .map(grammarFiles_1.loadGrammar)
            .map(async (def) => new grammar_1.Grammar(await def));
        const grammars = await Promise.all(pGrammars);
        return new Registry(grammars);
    }
}
exports.Registry = Registry;
//# sourceMappingURL=registry.js.map