import { GrammarDefinition, Pattern } from './grammarDefinition';
import { MatchOffsetResult } from './regexpUtil';
export interface TokenizeLineResult {
    tokens: Token[];
    state: TokenizerState;
}
export interface TokenizerState extends Rule {
    readonly depth: number;
}
export interface Token {
    readonly startIndex: number;
    readonly endIndex: number;
    readonly scopes: string[];
}
export interface TokenizeIterator extends IterableIterator<string> {
    next: (line: string) => IteratorResult<string>;
    [Symbol.iterator]: () => TokenizeIterator;
}
export interface Rule {
    readonly grammarDef: GrammarDefinition;
    readonly pattern: Pattern;
    readonly end?: RegExp;
    readonly parent?: Rule;
    readonly depth: number;
    readonly scope: string | undefined;
    readonly comment: string;
}
export interface MatchResult {
    match?: RegExpExecArray;
    rule: Rule;
}
export declare function tokenizeLine(text: string, rule: Rule): TokenizeLineResult;
export declare function matchRule(text: string, offset: number, rule: Rule): MatchResult;
export declare function mergeTokens(tokens: Token[]): Token[];
export interface CapturedSpan {
    captureGroups: string[];
    begin: number;
    end: number;
}
export declare function pairCaptureGroupsToMatchOffsets(captureGroups: string[], match: MatchOffsetResult): CapturedSpan[];
export declare function grammarToRule(grammar: GrammarDefinition, parent?: Rule): Rule;
