"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cacheMap_1 = require("../util/cacheMap");
const bgColor = '#202020';
const defaultColorApplicator = createDefaultColorApplicator();
const defaultColorDef = createDefaultColorMap(defaultColorApplicator);
function createScopeColorizer(colorDef = defaultColorDef) {
    const { colorMap, defaultColorizer } = colorDef;
    const scopeCache = cacheMap_1.create((scopes) => {
        for (const [reg, fn] of colorMap) {
            if (reg.test(scopes)) {
                return fn;
            }
        }
        return defaultColorizer;
    });
    function colorize(text, scopes) {
        const fn = scopeCache.get(scopes);
        return fn(text);
    }
    return colorize;
}
exports.createScopeColorizer = createScopeColorizer;
function createColorizer(scopeColorizer = createScopeColorizer(defaultColorDef)) {
    return function (text, tokens) {
        const parts = [];
        let pos = 0;
        for (const token of tokens) {
            if (token.startIndex > pos) {
                parts.push(scopeColorizer(text.slice(pos, token.startIndex), ''));
            }
            parts.push(scopeColorizer(text.slice(token.startIndex, token.endIndex), token.scopes.join(' ')));
            pos = token.endIndex;
        }
        if (pos < text.length) {
            parts.push(scopeColorizer(text.slice(pos), ''));
        }
        return parts.join('');
    };
}
exports.createColorizer = createColorizer;
function createDefaultColorMap(chalk) {
    const baseColor = chalk.bgHex(bgColor);
    const colorMap = [
        [/ keyword/, baseColor.yellow],
        [/ entity.name/, baseColor.blue],
        [/ variable/, baseColor.greenBright],
        [/ string/, baseColor.yellowBright],
        [/comment/, baseColor.dim.green],
        [/ punctuation/, baseColor.yellow],
        [/support.function/, baseColor.greenBright],
        [/^source/, baseColor.gray],
        [/^title$/, baseColor.dim],
    ];
    const defaultColorizer = baseColor.dim;
    return {
        colorMap,
        defaultColorizer,
    };
}
exports.createDefaultColorMap = createDefaultColorMap;
function createDefaultColorApplicator() {
    const applicator = ((...text) => text.join(''));
    applicator.red = applicator;
    applicator.green = applicator;
    applicator.yellow = applicator;
    applicator.blue = applicator;
    applicator.greenBright = applicator;
    applicator.yellowBright = applicator;
    applicator.dim = applicator;
    applicator.gray = applicator;
    applicator.bgHex = function () { return this; };
    return applicator;
}
exports.createDefaultColorApplicator = createDefaultColorApplicator;
//# sourceMappingURL=tokenColorizer.js.map